@@ -12,13 +12,15 @@ import boto3
 from botocore.exceptions import NoCredentialsError
 import pandas as pd
 
+from nemo_library.features.foxfile import FOXFile
+from nemo_library.features.foxmeta import FOXMeta
 from nemo_library.features.nemo_persistence_api import (
-    createImportedColumns,
-    getImportedColumns,
+    createColumns,
+    getColumns,
     getProjectID,
 )
 from nemo_library.features.nemo_persistence_api import createProjects
-from nemo_library.model.imported_column import ImportedColumn
+from nemo_library.model.column import Column
 from nemo_library.model.project import Project
 from nemo_library.utils.config import Config
 from nemo_library.utils.utils import (
@@ -43,12 +45,19 @@ def ReUploadDataFrame(
     format_data: bool = True,
 ) -> None:
 
+    # Default ImportConfigurations
     if import_configuration is None:
         import_configuration = ImportConfigurations()
 
+    # format data? we need to import first and then use the upload dataframe api
     if format_data:
         df = _format_data(df, import_configuration)
 
+    # check if project exists
+    if not getProjectID(config, projectname):
+        logging.info(f"Project {projectname} not found - create it")
+        createProjects(config=config, projects=[Project(displayName=projectname)])
+
     with tempfile.TemporaryDirectory() as temp_dir:
         temp_file_path = os.path.join(temp_dir, "tempfile.csv")
 
@@ -135,8 +144,18 @@ def ReUploadFile(
             df = pd.read_parquet(filename)
         elif ext in [".h5", ".hdf"]:
             df = pd.read_hdf(filename)
+        elif ext in [".fox"]:
+            foxfile = FOXFile(filename)
+            try:
+                df = foxfile.read()
+                meta = FOXMeta(foxfile)
+                meta.reconcile_metadata(config=config, projectname=projectname)
+            finally:
+                foxfile.close()
         else:
-            raise ValueError(f"Dateiformat {ext} wird nicht unterstützt.")
+            raise ValueError(
+                f"File format {ext} not supported. Please use .csv, .xls, .xlsx, .json, .parquet, .fox or .h5/.hdf files."
+            )
         ReUploadDataFrame(
             config=config,
             projectname=projectname,
@@ -188,7 +207,7 @@ def ReUploadFile(
         project_id = getProjectID(config, projectname)
         if not project_id:
             logging.info(f"Project {projectname} not found - create it")
-            createProjects(config=config,projects=[Project(displayName=projectname)])
+            createProjects(config=config, projects=[Project(displayName=projectname)])
             project_id = getProjectID(config, projectname)
 
         headers = config.connection_get_headers()
@@ -205,6 +224,7 @@ def ReUploadFile(
             ) as f_out:
                 shutil.copyfileobj(f_in, f_out)
         logging.info(f"File {filename} has been compressed to {gzipped_filename}")
+        time.sleep(1)
 
         # Retrieve temporary credentials from NEMO TVM
         response = requests.get(
@@ -296,6 +316,10 @@ def ReUploadFile(
                     headers=headers,
                     json=data,
                 )
+                if response.status_code != 200:
+                    raise Exception(
+                        f"Request failed. Status: {response.status_code}, error: {response.text}"
+                    )
                 resultjs = json.loads(response.text)
                 df = pd.json_normalize(resultjs["records"])
                 df_filtered = df[df["id"] == taskid]
@@ -347,6 +371,10 @@ def ReUploadFile(
                     headers=headers,
                     json=data,
                 )
+                if response.status_code != 200:
+                    raise Exception(
+                        f"Request failed. Status: {response.status_code}, error: {response.text}"
+                    )
                 resultjs = json.loads(response.text)
                 df = pd.json_normalize(resultjs["records"])
                 df_filtered = df[df["id"] == taskid]
@@ -429,7 +457,8 @@ def _format_data(
         return value
 
     str_columns = df.select_dtypes(include=["object", "string"]).columns
-    df[str_columns] = df[str_columns].map(escape_special_chars)
+    for col in str_columns:
+        df[col] = df[col].map(escape_special_chars)
 
     return df
 
@@ -460,8 +489,8 @@ def synchronizeCsvColsAndImportedColumns(
         - Creates new imported columns in the project for any CSV column names not already present.
         - Uses utility functions `display_name`, `internal_name`, and `import_name` to format column names.
     """
-    ics = getImportedColumns(config, projectname)
-    ics_internal = [ic.internalName for ic in ics]
+    cols = getColumns(config, projectname)
+    cols_internal = [col.internalName for col in cols]
 
     # Read the first line of the CSV file to get column names
     with open(filename, "r") as file:
@@ -476,20 +505,19 @@ def synchronizeCsvColsAndImportedColumns(
     for column_name in csv_display_names:
 
         # Check if the record with internal_name equal to the column name exists
-        if get_internal_name(column_name) in ics_internal:
+        if get_internal_name(column_name) in cols_internal:
             logging.info(f"Record found for column '{column_name}' in the DataFrame.")
         else:
             logging.info(
                 f"No record found for column '{column_name}' in the DataFrame - create it."
             )
             new_columns.append(
-                ImportedColumn(
-                    displayName=column_name,
-                    dataType="string",
+                Column(
+                    displayName=column_name, dataType="string", columnType="ExportedColumn"
                 )
             )
 
     if new_columns:
-        createImportedColumns(
-            config=config, projectname=projectname, importedcolumns=new_columns
+        createColumns(
+            config=config, projectname=projectname, columns=new_columns
         )
