@@ -1,25 +1,26 @@
 import json
 import logging
 import re
-from typing import Any, List, Type, TypeVar, get_type_hints
+from typing import Any, Type, TypeVar, get_type_hints
 
 import requests
 
 
 from nemo_library.model.application import Application
 from nemo_library.model.attribute_group import AttributeGroup
-from nemo_library.model.defined_column import DefinedColumn
+from nemo_library.model.attribute_link import AttributeLink
+from nemo_library.model.column import Column
 from nemo_library.model.dependency_tree import DependencyTree
 from nemo_library.model.diagram import Diagram
-from nemo_library.model.imported_column import ImportedColumn
 from nemo_library.model.metric import Metric
 from nemo_library.model.pages import Page
 from nemo_library.model.project import Project
 from nemo_library.model.report import Report
+from nemo_library.model.rule import Rule
 from nemo_library.model.subprocess import SubProcess
 from nemo_library.model.tile import Tile
 from nemo_library.utils.config import Config
-from nemo_library.utils.utils import FilterType, FilterValue, clean_meta_data, log_error
+from nemo_library.utils.utils import FilterType, FilterValue, log_error
 
 T = TypeVar("T")
 
@@ -52,7 +53,7 @@ def _deserializeMetaDataObject(value: Any, target_type: Type) -> Any:
 def _generic_metadata_create_or_update(
     config: Config,
     projectname: str,
-    objects: List[T],
+    objects: list[T],
     endpoint: str,
     get_existing_func,
 ) -> None:
@@ -69,9 +70,11 @@ def _generic_metadata_create_or_update(
     # Initialize request
     headers = config.connection_get_headers()
     project_id = getProjectID(config, projectname)
-
+    params = {"translationHandling": "UseAuxiliaryTranslationFields"}
     for obj in objects:
-        logging.info(f"Create/update {endpoint} '{obj.displayName}'")
+        logging.info(
+            f"Create/update {endpoint[:-1] if endpoint.endswith("s") else endpoint} '{obj.displayName if hasattr(obj, 'displayName') else obj.internalName}'"
+        )
 
         obj.tenant = config.get_tenant()
         obj.projectId = project_id
@@ -92,10 +95,11 @@ def _generic_metadata_create_or_update(
                 f"{config.get_config_nemo_url()}/api/nemo-persistence/metadata/{endpoint}/{obj.id}",
                 json=obj.to_dict(),
                 headers=headers,
+                params=params,
             )
             if response.status_code != 200:
                 log_error(
-                    f"Request failed.\nURL: {f"{config.get_config_nemo_url()}/api/nemo-persistence/metadata/{endpoint}/{obj.id}",}\nStatus: {response.status_code}, error: {response.text}"
+                    f"PUT Request failed.\nURL: {f"{config.get_config_nemo_url()}/api/nemo-persistence/metadata/{endpoint}/{obj.id}"}\nobject: {json.dumps(obj.to_dict())}\nStatus: {response.status_code}, error: {response.text}"
                 )
 
         else:
@@ -104,14 +108,15 @@ def _generic_metadata_create_or_update(
                 f"{config.get_config_nemo_url()}/api/nemo-persistence/metadata/{endpoint}",
                 json=obj.to_dict(),
                 headers=headers,
+                params=params,
             )
             if response.status_code != 201:
                 log_error(
-                    f"Request failed.\nURL: {f"{config.get_config_nemo_url()}/api/nemo-persistence/metadata/{endpoint}"}\nStatus: {response.status_code}, error: {response.text}"
+                    f"POST Request failed.\nURL: {f"{config.get_config_nemo_url()}/api/nemo-persistence/metadata/{endpoint}"}\nobject: {json.dumps(obj.to_dict())}\nStatus: {response.status_code}, error: {response.text}"
                 )
 
 
-def _generic_metadata_delete(config: Config, ids: List[str], endpoint: str) -> None:
+def _generic_metadata_delete(config: Config, ids: list[str], endpoint: str) -> None:
     """
     Generic function to delete metadata entries.
 
@@ -124,16 +129,19 @@ def _generic_metadata_delete(config: Config, ids: List[str], endpoint: str) -> N
     headers = config.connection_get_headers()
 
     for obj_id in ids:
-        logging.info(f"Deleting {endpoint[:-1]} with ID {obj_id}")
+        logging.info(
+            f"Deleting {endpoint[:-1] if endpoint.endswith("s") else endpoint} with ID {obj_id}"
+        )
 
         response = requests.delete(
             f"{config.get_config_nemo_url()}/api/nemo-persistence/metadata/{endpoint}/{obj_id}",
             headers=headers,
+            params={"translationHandling": "UseAuxiliaryTranslationFields"},
         )
 
         if response.status_code != 204:
             log_error(
-                f"Request failed.\nURL: {f"{config.get_config_nemo_url()}/api/nemo-persistence/metadata/{endpoint}/{obj_id}"}\nStatus: {response.status_code}, error: {response.text}"
+                f"DELETE Request failed.\nURL: {f"{config.get_config_nemo_url()}/api/nemo-persistence/metadata/{endpoint}/{obj_id}"}\nStatus: {response.status_code}, error: {response.text}"
             )
 
 
@@ -146,7 +154,7 @@ def _generic_metadata_get(
     filter: str = "*",
     filter_type: FilterType = FilterType.STARTSWITH,
     filter_value: FilterValue = FilterValue.DISPLAYNAME,
-) -> List[T]:
+) -> list[T]:
     """
     Generic method to fetch and filter metadata for different objects.
 
@@ -163,15 +171,17 @@ def _generic_metadata_get(
     # Initialize request
     headers = config.connection_get_headers()
     project_id = getProjectID(config, projectname)
+    params = {"translationHandling": "UseAuxiliaryTranslationFields"}
 
     response = requests.get(
         f"{config.get_config_nemo_url()}/api/nemo-persistence/metadata/{endpoint}/project/{project_id}{endpoint_postfix}",
         headers=headers,
+        params=params,
     )
 
     if response.status_code != 200:
         log_error(
-            f"Request failed.\nURL:{f"{config.get_config_nemo_url()}/api/nemo-persistence/metadata/{endpoint}/project/{project_id}{endpoint_postfix}"}\nStatus: {response.status_code}, error: {response.text}"
+            f"GET Request failed.\nURL:{f"{config.get_config_nemo_url()}/api/nemo-persistence/metadata/{endpoint}/project/{project_id}{endpoint_postfix}"}\nStatus: {response.status_code}, error: {response.text}"
         )
         return []
 
@@ -200,9 +210,7 @@ def _generic_metadata_get(
         if match_filter(item.get(filter_value.value, ""), filter, filter_type)
     ]
 
-    # Clean metadata and return the list of objects
-    cleaned_data = clean_meta_data(filtered_data)
-    return [_deserializeMetaDataObject(item, return_type) for item in cleaned_data]
+    return [_deserializeMetaDataObject(item, return_type) for item in filtered_data]
 
 
 def getProjectID(
@@ -244,7 +252,7 @@ def getAttributeGroups(
     filter: str = "*",
     filter_type: FilterType = FilterType.STARTSWITH,
     filter_value: FilterValue = FilterValue.DISPLAYNAME,
-) -> List[AttributeGroup]:
+) -> list[AttributeGroup]:
     """Fetches AttributeGroups metadata with the given filters."""
     return _generic_metadata_get(
         config,
@@ -258,13 +266,34 @@ def getAttributeGroups(
     )
 
 
+def getAttributeLinks(
+    config: Config,
+    projectname: str,
+    filter: str = "*",
+    filter_type: FilterType = FilterType.STARTSWITH,
+    filter_value: FilterValue = FilterValue.DISPLAYNAME,
+) -> list[AttributeLink]:
+    """Fetches AttributeLinks metadata with the given filters."""
+
+    return _generic_metadata_get(
+        config,
+        projectname,
+        "AttributeLink",
+        "",
+        AttributeLink,
+        filter,
+        filter_type,
+        filter_value,
+    )
+
+
 def getMetrics(
     config: Config,
     projectname: str,
     filter: str = "*",
     filter_type: FilterType = FilterType.STARTSWITH,
     filter_value: FilterValue = FilterValue.DISPLAYNAME,
-) -> List[Metric]:
+) -> list[Metric]:
     """Fetches Metrics metadata with the given filters."""
     return _generic_metadata_get(
         config, projectname, "Metrics", "", Metric, filter, filter_type, filter_value
@@ -277,7 +306,7 @@ def getTiles(
     filter: str = "*",
     filter_type: FilterType = FilterType.STARTSWITH,
     filter_value: FilterValue = FilterValue.DISPLAYNAME,
-) -> List[Tile]:
+) -> list[Tile]:
     """Fetches Tiles metadata with the given filters."""
     return _generic_metadata_get(
         config, projectname, "Tiles", "", Tile, filter, filter_type, filter_value
@@ -290,7 +319,7 @@ def getPages(
     filter: str = "*",
     filter_type: FilterType = FilterType.STARTSWITH,
     filter_value: FilterValue = FilterValue.DISPLAYNAME,
-) -> List[Page]:
+) -> list[Page]:
     """Fetches Pages metadata with the given filters."""
     return _generic_metadata_get(
         config, projectname, "Pages", "", Page, filter, filter_type, filter_value
@@ -303,7 +332,7 @@ def getApplications(
     filter: str = "*",
     filter_type: FilterType = FilterType.STARTSWITH,
     filter_value: FilterValue = FilterValue.DISPLAYNAME,
-) -> List[Application]:
+) -> list[Application]:
     """Fetches Applications metadata with the given filters."""
     return _generic_metadata_get(
         config,
@@ -323,7 +352,7 @@ def getDiagrams(
     filter: str = "*",
     filter_type: FilterType = FilterType.STARTSWITH,
     filter_value: FilterValue = FilterValue.DISPLAYNAME,
-) -> List[Diagram]:
+) -> list[Diagram]:
     """Fetches Diagrams metadata with the given filters."""
     return _generic_metadata_get(
         config,
@@ -337,40 +366,40 @@ def getDiagrams(
     )
 
 
-def getDefinedColumns(
+def getColumns(
     config: Config,
     projectname: str,
     filter: str = "*",
     filter_type: FilterType = FilterType.STARTSWITH,
     filter_value: FilterValue = FilterValue.DISPLAYNAME,
-) -> List[DefinedColumn]:
-    """Fetches DefinedColumns metadata with the given filters."""
+) -> list[Column]:
+    """Fetches columns metadata with the given filters."""
     return _generic_metadata_get(
         config,
         projectname,
         "Columns",
-        "/defined",
-        DefinedColumn,
+        "/all",
+        Column,
         filter,
         filter_type,
         filter_value,
     )
 
 
-def getImportedColumns(
+def getReports(
     config: Config,
     projectname: str,
     filter: str = "*",
     filter_type: FilterType = FilterType.STARTSWITH,
     filter_value: FilterValue = FilterValue.DISPLAYNAME,
-) -> List[ImportedColumn]:
-    """Fetches ImportedColumns metadata with the given filters."""
+) -> list[Report]:
+    """Fetches Reports metadata with the given filters."""
     return _generic_metadata_get(
         config,
         projectname,
-        "Columns",
-        "/exported",
-        ImportedColumn,
+        "Reports",
+        "/reports",
+        Report,
         filter,
         filter_type,
         filter_value,
@@ -383,7 +412,7 @@ def getSubProcesses(
     filter: str = "*",
     filter_type: FilterType = FilterType.STARTSWITH,
     filter_value: FilterValue = FilterValue.DISPLAYNAME,
-) -> List[SubProcess]:
+) -> list[SubProcess]:
     """Fetches SubProcesss metadata with the given filters."""
     return _generic_metadata_get(
         config,
@@ -397,117 +426,201 @@ def getSubProcesses(
     )
 
 
-def deleteDefinedColumns(config: Config, definedcolumns: List[str]) -> None:
-    """Deletes a list of Defined Columns by their IDs."""
-    _generic_metadata_delete(config, definedcolumns, "Columns")
+def getRules(
+    config: Config,
+    projectname: str,
+    filter: str = "*",
+    filter_type: FilterType = FilterType.STARTSWITH,
+    filter_value: FilterValue = FilterValue.DISPLAYNAME,
+) -> list[Rule]:
+    """Fetches Rules metadata with the given filters."""
+    return _generic_metadata_get(
+        config, projectname, "Rule", "/rules", Rule, filter, filter_type, filter_value
+    )
 
 
-def deleteImportedColumns(config: Config, importedcolumns: List[str]) -> None:
-    """Deletes a list of Imported Columns by their IDs."""
-    _generic_metadata_delete(config, importedcolumns, "Columns")
+def deleteColumns(config: Config, columns: list[str]) -> None:
+    """Deletes a list of Columns by their IDs."""
+    _generic_metadata_delete(config, columns, "Columns")
 
 
-def deleteMetrics(config: Config, metrics: List[str]) -> None:
+def deleteMetrics(config: Config, metrics: list[str]) -> None:
     """Deletes a list of Metrics by their IDs."""
     _generic_metadata_delete(config, metrics, "Metrics")
 
 
-def deleteTiles(config: Config, tiles: List[str]) -> None:
+def deleteTiles(config: Config, tiles: list[str]) -> None:
     """Deletes a list of Tiles by their IDs."""
     _generic_metadata_delete(config, tiles, "Tiles")
 
 
-def deleteAttributeGroups(config: Config, attributegroups: List[str]) -> None:
+def deleteAttributeGroups(config: Config, attributegroups: list[str]) -> None:
     """Deletes a list of AttributeGroups by their IDs."""
     _generic_metadata_delete(config, attributegroups, "AttributeGroup")
 
 
-def deletePages(config: Config, pages: List[str]) -> None:
+def deleteAttributeLinks(config: Config, attributelinks: list[str]) -> None:
+    """Deletes a list of AttributeLinks by their IDs."""
+    _generic_metadata_delete(config, attributelinks, "AttributeLink")
+
+
+def deletePages(config: Config, pages: list[str]) -> None:
     """Deletes a list of Pages by their IDs."""
     _generic_metadata_delete(config, pages, "Pages")
 
 
-def deleteApplications(config: Config, applications: List[str]) -> None:
+def deleteApplications(config: Config, applications: list[str]) -> None:
     """Deletes a list of Pages by their IDs."""
     _generic_metadata_delete(config, applications, "Applications")
 
 
-def deleteDiagrams(config: Config, diagrams: List[str]) -> None:
+def deleteDiagrams(config: Config, diagrams: list[str]) -> None:
     """Deletes a list of Diagrams by their IDs."""
     _generic_metadata_delete(config, diagrams, "Diagrams")
 
 
-def deleteSubprocesses(config: Config, subprocesses: List[str]) -> None:
+def deleteSubprocesses(config: Config, subprocesses: list[str]) -> None:
     """Deletes a list of SubProcesses by their IDs."""
     _generic_metadata_delete(config, subprocesses, "SubProcess")
 
 
-def createDefinedColumns(
-    config: Config, projectname: str, definedcolumns: List[DefinedColumn]
-) -> None:
-    """Creates or updates a list of DefinedColumns."""
+def deleteReports(config: Config, reports: list[str]) -> None:
+    """Deletes a list of Reports by their IDs."""
+    _generic_metadata_delete(config, reports, "Reports")
+
+
+def deleteRules(config: Config, rules: list[str]) -> None:
+    """Deletes a list of Rules by their IDs."""
+    _generic_metadata_delete(config, rules, "Rule")
+
+
+def createColumns(config: Config, projectname: str, columns: list[Column]) -> None:
+    """Creates or updates a list of columns."""
     _generic_metadata_create_or_update(
-        config, projectname, definedcolumns, "Columns", getDefinedColumns
+        config=config,
+        projectname=projectname,
+        objects=columns,
+        endpoint="Columns",
+        get_existing_func=getColumns,
     )
 
 
-def createImportedColumns(
-    config: Config, projectname: str, importedcolumns: List[ImportedColumn]
-) -> None:
-    """Creates or updates a list of ImportedColumns."""
-    _generic_metadata_create_or_update(
-        config, projectname, importedcolumns, "Columns", getImportedColumns
-    )
-
-
-def createMetrics(config: Config, projectname: str, metrics: List[Metric]) -> None:
+def createMetrics(config: Config, projectname: str, metrics: list[Metric]) -> None:
     """Creates or updates a list of Metrics."""
     _generic_metadata_create_or_update(
-        config, projectname, metrics, "Metrics", getMetrics
+        config=config,
+        projectname=projectname,
+        objects=metrics,
+        endpoint="Metrics",
+        get_existing_func=getMetrics,
     )
 
 
-def createTiles(config: Config, projectname: str, tiles: List[Tile]) -> None:
+def createTiles(config: Config, projectname: str, tiles: list[Tile]) -> None:
     """Creates or updates a list of Tiles."""
-    _generic_metadata_create_or_update(config, projectname, tiles, "Tiles", getTiles)
+    _generic_metadata_create_or_update(
+        config=config,
+        projectname=projectname,
+        objects=tiles,
+        endpoint="Tiles",
+        get_existing_func=getTiles,
+    )
 
 
 def createAttributeGroups(
-    config: Config, projectname: str, attributegroups: List[AttributeGroup]
+    config: Config, projectname: str, attributegroups: list[AttributeGroup]
 ) -> None:
     """Creates or updates a list of AttributeGroups."""
     _generic_metadata_create_or_update(
-        config, projectname, attributegroups, "AttributeGroup", getAttributeGroups
+        config=config,
+        projectname=projectname,
+        objects=attributegroups,
+        endpoint="AttributeGroup",
+        get_existing_func=getAttributeGroups,
     )
 
 
-def createPages(config: Config, projectname: str, pages: List[Page]) -> None:
+def createAttributeLinks(
+    config: Config, projectname: str, attributelinks: list[AttributeLink]
+) -> None:
+    """Creates or updates a list of AttributeLinks."""
+
+    _generic_metadata_create_or_update(
+        config=config,
+        projectname=projectname,
+        objects=attributelinks,
+        endpoint="AttributeLink",
+        get_existing_func=getAttributeLinks,
+    )
+
+
+def createPages(config: Config, projectname: str, pages: list[Page]) -> None:
     """Creates or updates a list of Pages."""
-    _generic_metadata_create_or_update(config, projectname, pages, "Pages", getPages)
+    _generic_metadata_create_or_update(
+        config=config,
+        projectname=projectname,
+        objects=pages,
+        endpoint="Pages",
+        get_existing_func=getPages,
+    )
 
 
 def createApplications(
-    config: Config, projectname: str, applications: List[Application]
+    config: Config, projectname: str, applications: list[Application]
 ) -> None:
     """Creates or updates a list of Applications."""
     _generic_metadata_create_or_update(
-        config, projectname, applications, "Applications", getApplications
+        config=config,
+        projectname=projectname,
+        objects=applications,
+        endpoint="Applications",
+        get_existing_func=getApplications,
     )
 
 
-def createDiagrams(config: Config, projectname: str, diagrams: List[Diagram]) -> None:
+def createDiagrams(config: Config, projectname: str, diagrams: list[Diagram]) -> None:
     """Creates or updates a list of Diagrams."""
     _generic_metadata_create_or_update(
-        config, projectname, diagrams, "Diagrams", getDiagrams
+        config=config,
+        projectname=projectname,
+        objects=diagrams,
+        endpoint="Diagrams",
+        get_existing_func=getDiagrams,
     )
 
 
 def createSubProcesses(
-    config: Config, projectname: str, subprocesses: List[SubProcess]
+    config: Config, projectname: str, subprocesses: list[SubProcess]
 ) -> None:
     """Creates or updates a list of SubProcesses."""
     _generic_metadata_create_or_update(
-        config, projectname, subprocesses, "SubProcess", getSubProcesses
+        config=config,
+        projectname=projectname,
+        objects=subprocesses,
+        endpoint="SubProcess",
+        get_existing_func=getSubProcesses,
+    )
+
+
+def createReports(config: Config, projectname: str, reports: list[Report]) -> None:
+    """Creates or updates a list of Reports."""
+    _generic_metadata_create_or_update(
+        config=config,
+        projectname=projectname,
+        objects=reports,
+        endpoint="Reports",
+        get_existing_func=getReports,
+    )
+
+
+def createRules(config: Config, projectname: str, rules: list[Rule]) -> None:
+    """Creates or updates a list of Rules."""
+    _generic_metadata_create_or_update(
+        config=config,
+        projectname=projectname,
+        objects=rules,
+        endpoint="Rule",
+        get_existing_func=getRules,
     )
 
 
@@ -516,7 +629,7 @@ def getProjects(
     filter: str = "*",
     filter_type: FilterType = FilterType.STARTSWITH,
     filter_value: FilterValue = FilterValue.DISPLAYNAME,
-) -> List[Project]:
+) -> list[Project]:
     """Fetches Projects metadata with the given filters."""
 
     # cannot use the generic meta data getter, since this is "above" the other object
@@ -555,17 +668,15 @@ def getProjects(
         if match_filter(item.get(filter_value.value, ""), filter, filter_type)
     ]
 
-    # Clean metadata and return the list of objects
-    cleaned_data = clean_meta_data(filtered_data)
-    return [_deserializeMetaDataObject(item, Project) for item in cleaned_data]
+    return [_deserializeMetaDataObject(item, Project) for item in filtered_data]
 
 
-def deleteProjects(config: Config, projects: List[str]) -> None:
+def deleteProjects(config: Config, projects: list[str]) -> None:
     """Deletes a list of projects by their IDs."""
     _generic_metadata_delete(config, projects, "Project")
 
 
-def createProjects(config: Config, projects: List[Project]) -> None:
+def createProjects(config: Config, projects: list[Project]) -> None:
     """Creates or updates a list of Projects."""
     # Initialize request
     headers = config.connection_get_headers()
@@ -593,7 +704,7 @@ def createProjects(config: Config, projects: List[Project]) -> None:
             )
             if response.status_code != 200:
                 log_error(
-                    f"Request failed.\nURL: {f"{config.get_config_nemo_url()}/api/nemo-projects/projects/{project.id}"}\nStatus: {response.status_code}, error: {response.text}"
+                    f"PUT Request failed.\nURL: {f"{config.get_config_nemo_url()}/api/nemo-projects/projects/{project.id}"}\nobject: {json.dumps(project.to_dict())}\nStatus: {response.status_code}, error: {response.text}"
                 )
 
         else:
@@ -605,7 +716,7 @@ def createProjects(config: Config, projects: List[Project]) -> None:
             )
             if response.status_code != 201:
                 log_error(
-                    f"Request failed.\nURL: {f"{config.get_config_nemo_url()}/api/nemo-persistence/metadata/Project"}\nStatus: {response.status_code}, error: {response.text}"
+                    f"POST Request failed.\nURL: {f"{config.get_config_nemo_url()}/api/nemo-persistence/metadata/Project"}\nobject: {json.dumps(project.to_dict())}\nStatus: {response.status_code}, error: {response.text}"
                 )
 
 
@@ -628,35 +739,3 @@ def getDependencyTree(config: Config, id: str) -> DependencyTree:
 
     data = json.loads(response.text)
     return DependencyTree.from_dict(data)
-
-
-def getReports(
-    config: Config,
-    projectname: str,
-    filter: str = "*",
-    filter_type: FilterType = FilterType.STARTSWITH,
-    filter_value: FilterValue = FilterValue.DISPLAYNAME,
-) -> List[Report]:
-    """Fetches Reports metadata with the given filters."""
-    return _generic_metadata_get(
-        config,
-        projectname,
-        "Reports",
-        "/reports",
-        Report,
-        filter,
-        filter_type,
-        filter_value,
-    )
-
-
-def createReports(config: Config, projectname: str, reports: List[Report]) -> None:
-    """Creates or updates a list of Reports."""
-    _generic_metadata_create_or_update(
-        config, projectname, reports, "Reports", getReports
-    )
-
-
-def deleteReports(config: Config, reports: List[str]) -> None:
-    """Deletes a list of Reports by their IDs."""
-    _generic_metadata_delete(config, reports, "Reports")
