@@ -1,5 +1,4 @@
 from dataclasses import dataclass, asdict, field
-from typing import Dict
 
 from nemo_library.utils.utils import get_internal_name
 
@@ -9,18 +8,21 @@ class AttributeGroup:
     """
     Represents a group of attributes with various properties and settings.
     """
+
     attributeGroupType: str = "Standard"
     defaultMetricGroup: bool = False
     defaultDefinedColumnGroup: bool = False
-    displayName: str = None
-    displayNameTranslations: Dict[str, str] = field(default_factory=dict)
+    displayName: str | None = None
+    displayNameTranslations: dict[str, str] = field(default_factory=dict)
     isCoupled: bool = False
-    focusOrder: str = ""
     internalName: str = None
-    parentAttributeGroupInternalName: str = ""
+    parentAttributeGroupInternalName: str | None = None
     id: str = ""
     projectId: str = ""
     tenant: str = ""
+    isCustom: bool = False
+    metadataClassificationInternalName: str = ""
+    order: str = ""
 
     def to_dict(self):
         """
