@@ -1,22 +1,23 @@
-from dataclasses import dataclass, field, asdict
-from typing import Dict, Optional
+from dataclasses import dataclass, asdict
+
 
 @dataclass
 class Tile:
     """
     Represents a Tile with various attributes related to its display and metadata.
     """
+
     aggregation: str
     description: str
-    descriptionTranslations: Dict[str, str]
+    descriptionTranslations: dict[str, str]
     displayName: str
-    displayNameTranslations: Dict[str, str]
+    displayNameTranslations: dict[str, str]
     frequency: str
     graphic: str
     internalName: str
     status: str
     tileGroup: str
-    tileGroupTranslations: Dict[str, str]
+    tileGroupTranslations: dict[str, str]
     tileSourceID: str
     tileSourceInternalName: str
     type: str
@@ -24,6 +25,8 @@ class Tile:
     id: str
     projectId: str
     tenant: str
+    isCustom: bool = False
+    metadataClassificationInternalName: str = ""
 
     def to_dict(self):
         """
@@ -33,4 +36,3 @@ class Tile:
             dict: A dictionary representation of the Tile instance.
         """
         return asdict(self)
-
