@@ -1,20 +1,10 @@
-import pytest
-
-from nemo_library import NemoLibrary
-from datetime import datetime
-
-from nemo_library.model.imported_column import ImportedColumn
+from nemo_library.model.column import Column
 from nemo_library.model.project import Project
+from tests.testutils import getNL
 
 HS_PROJECT_NAME = "gs_unit_test_HubSpot"
 
 
-def getNL():
-    return NemoLibrary(
-        config_file="tests/config.ini",
-    )
-
-
 def test_FetchDealFromHubSpotAndUploadToNEMO():
     nl = getNL()
 
@@ -27,13 +17,19 @@ def test_FetchDealFromHubSpotAndUploadToNEMO():
         [Project(displayName=HS_PROJECT_NAME, description="project for unit tests")]
     )
     new_columns = []
-    new_columns.append(ImportedColumn(displayName="deal_id", dataType="float"))
     new_columns.append(
-        ImportedColumn(displayName="update_closedate_new_value", dataType="date")
+        Column(displayName="deal_id", dataType="float", columnType="ExportedColumn")
     )
-    nl.createImportedColumns(
+    new_columns.append(
+        Column(
+            displayName="update_closedate_new_value",
+            dataType="date",
+            columnType="ExportedColumn",
+        )
+    )
+    nl.createColumns(
         projectname=HS_PROJECT_NAME,
-        importedcolumns=new_columns,
+        columns=new_columns,
     )
     nl.setProjectMetaData(
         projectname=HS_PROJECT_NAME,
